/**
 * @file
 * @author  Caleb Fangmeier <caleb@fangmeier.tech>
 * @version 0.1
 *
 * @section LICENSE
 *
 *
 * MIT License
 *
 * Copyright (c) 2017 Caleb Fangmeier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef config_hpp
#define config_hpp
#include <string>
#include <iostream>

#include "yaml-cpp/yaml.h"

namespace fv::util{

class Config{
    private:
        std::string input_filename;
        YAML::Node root;
    public:
        Config ()
          :root(), input_filename("") { }
        Config (const Config& c)
          :root(c.root), input_filename(c.input_filename) { }
        Config (const std::string& input_filename)
          :root(YAML::LoadFile(input_filename)),
           input_filename(input_filename) { }

        template <typename KEY_TYPE>
        YAML::Node get(const KEY_TYPE& key){
            auto val = root[key];
            return val;
        }
};

Config* the_config = nullptr;
void init_config(const std::string& input_filename){
    if(the_config) delete the_config;
    the_config = new Config(input_filename);
}

}
#endif // config_hpp
